/**
 * @file
 * Attaches next behavior for keyboard functionality.
 */

(function (Drupal, drupalSettings) {
  const elemName = 'keyboard-accessibility';
  const { size, type, color, outlineColor, boxSize } =
    drupalSettings.a11y.keyboard.border;

  let init = false;

  Drupal.behaviors.a11y_keyboard = {
    attach(context, settings) {
      const behavior = this;

      if (!init) {
        init = true;

        behavior.init();

        document.addEventListener('keyup', (e) => {
          behavior.show();
          behavior.keyUp(e);
        });
        document.addEventListener('keypress', (e) => {
          behavior.show();
          behavior.keyPress(e);
        });
        document.addEventListener('mousedown', behavior.hide);
        document.addEventListener('scroll', behavior.hide);
        document.addEventListener('keydown', behavior.hide);
      }
    },

    detach() {
      const behavior = this;

      document.removeEventListener('keyup', behavior.keyUp);
      document.removeEventListener('keypress', behavior.keyPress);
      document.removeEventListener('mousedown', behavior.hide);
      document.removeEventListener('scroll', behavior.hide);
      document.removeEventListener('keydown', behavior.hide);
    },

    init() {
      const container = document.createElement('div');
      container.setAttribute('class', elemName);
      container.style.display = 'none';
      container.style.border = `${size}px ${type} ${color}`;
      container.style.outline = `${size}px ${type} ${outlineColor}`;
      document.body.append(container);
    },

    hide(e) {
      const container = document.querySelector(`.${elemName}`);
      if ([1, 3, undefined].includes(e.which)) {
        container.style.display = 'none';
      }
    },

    show() {
      const container = document.querySelector(`.${elemName}`);
      container.style.display = 'block';
    },

    keyUp(e) {
      if (e.which === 9 || e.key === 'Tab') {
        this.onTabulate();
      }
    },

    keyPress(e) {
      if (e.which === 9 || e.key === 'Tab') {
        this.onTabulate();
      }
    },

    onTabulate() {
      const container = document.querySelector(`.${elemName}`);
      const active = document.activeElement;
      let clientRect = active.getBoundingClientRect();

      let childrenElement = active;
      let parentElement = active;
      let hasChildren = true;

      let width = active.offsetWidth || 0;
      let height = active.offsetHeight || 0;

      while (width === 0 || height === 0) {
        if (childrenElement.hasChildNodes() && hasChildren) {
          childrenElement = childrenElement.childNodes[0];
          if (childrenElement.tagName) {
            clientRect = childrenElement.getBoundingClientRect();
          }
          width = childrenElement.offsetWidth || 0;
          height = childrenElement.offsetHeight || 0;
        } else {
          hasChildren = false;
          parentElement = parentElement.parentElement;
          if (parentElement.tagName) {
            clientRect = parentElement.getBoundingClientRect();
          }
          width = parentElement.offsetWidth || 0;
          height = parentElement.offsetHeight || 0;
        }
      }

      const left = clientRect.left || 0;
      const top = clientRect.top || 0;

      container.style.left = `${left - boxSize / 4}px`;
      container.style.top = `${top - boxSize / 4}px`;
      container.style.width = `${width + boxSize / 2}px`;
      container.style.height = `${height + boxSize / 2}px`;
    },
  };
})(Drupal, drupalSettings);
