<?php

namespace Drupal\a11y\Plugin;

use Drupal\a11y\Entity\A11yInterface;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Provides the A11y plugin plugin manager.
 */
class A11yPluginManager extends DefaultPluginManager implements A11yPluginManagerInterface {

  use MessengerTrait, StringTranslationTrait;

  /**
   * The entity-type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  private $languageManager;

  /**
   * The module settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface|null
   */
  protected $loggerFactory;

  public function __construct(
    \Traversable $namespaces,
    CacheBackendInterface $cache_backend,
    ModuleHandlerInterface $module_handler,
    ?EntityTypeManagerInterface $entity_type_manager = NULL,
    ?LoggerChannelFactoryInterface $logger_factory = NULL,
  ) {
    parent::__construct('Plugin/A11yPlugin', $namespaces, $module_handler, 'Drupal\a11y\Plugin\A11yPluginInterface', 'Drupal\a11y\Annotation\A11yPlugin');

    $this->alterInfo('a11y_a11y_plugin_info');
    $this->setCacheBackend($cache_backend, 'a11y_a11y_plugin_plugins');

    // Initialize optional dependencies if provided.
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getAvailablePlugins(): array {
    $pluginAsListOption = [];

    try {
      // Use the injected entity type manager if available.
      if ($this->entityTypeManager) {
        $storage = $this->entityTypeManager->getStorage('a11y');
        $query = $storage->getQuery()
          ->accessCheck(FALSE)
          ->condition('status', TRUE);
        $ids = $query->execute();
        $plugins = $storage->loadMultiple($ids);
        foreach ($plugins as $key => $plugin) {
          $pluginAsListOption[$key] = $plugin->label();
        }
      }
    }
    catch (\Exception $e) {
      // Use the injected logger factory if available.
      if ($this->loggerFactory) {
        $this->loggerFactory->get('a11y')->error('An error occurred: @message', [
          '@message' => $e->getMessage(),
          'exception' => $e,
        ]);
      }
    }

    return $pluginAsListOption;
  }

  /**
   * {@inheritdoc}
   */
  public function createInstanceFromPlugin(A11yInterface $plugin): A11YPluginInterface {
    if ($plugin->getPluginType() === NULL) {
      throw new PluginException($this->t('The implementation %label has no plugin set yet.', [
        '%label' => $plugin->label(),
      ]));
    }

    $configuration = $plugin->toArray();
    $configuration['implementation'] = $configuration['id'];
    // Unset non-configuration plugin keys.
    unset($configuration['id'], $configuration['plugin_type'], $configuration['dependencies']);
    /** @var \Drupal\a11y\Plugin\A11YPluginInterface $plugin */
    $plugin = $this->createInstance($plugin->getPluginType(), $configuration);

    return $plugin;
  }

  /**
   * Get plugins definitions as options.
   *
   * Typically to be used as select/checkboxes/radios element options.
   *
   * @return array
   *   An array of id => label pairs of plugins.
   */
  public function getPluginsAsOptions() {
    $definitions = $this->getDefinitions();
    $plugins = array_column($definitions, 'label', 'id');
    asort($plugins);

    return $plugins;
  }

  /**
   * Get the provider for a given plugin.
   *
   * @param \Drupal\a11y\Entity\A11yInterface $plugin
   *   The A11y plugin entity.
   *
   * @return string
   *   The provider of the plugin.
   */
  public function getProviderFromPlugin(A11yInterface $plugin): string {
    /** @var \Drupal\a11y\Plugin\A11yPluginInterface $instance */
    $instance = self::createInstanceFromPlugin($plugin);
    return $instance->getPluginDefinition()['provider'];
  }

}
